############################################
## Supplementary material for the paper
## "Using Weighted Entropy to Rank Chemicals
## in Quantitative High Throughput Screening
## Experiments"
##
## written by Keith R. Shockley, 07-26-13
############################################

# preliminaries
"%w/o%" <- function(x, y) x[!x %in% y] #--  x without y

# define WES score function, to calculate WES score for each chemical
# "data" is the data file containing the concentration-response data
# (i.e., chemicals in rows of data file, responses for each concentration are in selected columns of data file)
# "RespCols" are the columns containing the responses (e.g., if the first 14 columns contain responses)
# "DetLimit" is the assay-specific detection limit (e.g., 25% - see paper)

WES.FX <- function(data, RespCols=1:14, DetLimit=25){

    WES=vector();
    for(i in 1:nrow(data)){
        pos.prim.na.resp = which(is.na(data[i,RespCols])); 
        pos.prim.na = sort(unique(pos.prim.na.resp));

        Responses = as.numeric(data[i,RespCols %w/o% c(RespCols)[pos.prim.na]]);   
        AbsResp = abs(Responses);

        Nu_WES = length(Responses)

        #calculate the component of entropy for each response. Note that 0 * log2(0) = 0 is assumed here, 
        #since no entropy component is calculated when the response is "0"
        if(Nu_WES != 0){
            WES.pre=vector();
            for(k in 1:length(AbsResp)){
                if(AbsResp[k] > 0 & AbsResp[k] >= DetLimit){
                    WES.pre[k] = -AbsResp[k]/sum(AbsResp)*log2(AbsResp[k]/sum(AbsResp))
                }
                if(AbsResp[k] > 0 & AbsResp[k] < DetLimit){
                    WES.pre[k]  = -(AbsResp[k]/DetLimit)/sum(AbsResp)*log2((AbsResp[k]/DetLimit)/sum(AbsResp))
                }
            }
            WES[i] = sum(WES.pre, na.rm=TRUE)
        }

        if(i/500 == as.integer(i/500)){print(paste("finished row", i))}
        }

       calls   <- WES;
       calls
}


##################################################################################
# Example 1: Calculate WES scores for illustrative profiles (see Table 1 in paper)
##################################################################################

chemResp.mat = matrix(NA, nrow=6, ncol=4);
chemResp.mat = data.frame(chemResp.mat);
chemResp.mat[1,] = c(75, 75, 75, 75);
chemResp.mat[2,] = c(10, 10, 10, 10);
chemResp.mat[3,] = c(30, 30, 70, 100);
chemResp.mat[4,] = c(-30, 30, -70, 100);
chemResp.mat[5,] = c(5, 5, 5, 85);
chemResp.mat[6,] = c(0, 0, 0, 100);

WES_Table1 = WES.FX(data=chemResp.mat, RespCols = 1:4);
WES_Table1
"[1] 2.0000000 0.2657542 1.8113645 1.8113645 0.2530902 0.0000000"


####################################################################################
# Example 2: Calculate WES scores for data generated according to the four-parameter
# Hill Equation model (see Figure 2 in paper)
####################################################################################

# generate data for RMAX=100, R0=0, HILL=1, error=25%
# increase AC50 by 0.01 from 10^-5 to 10^2
RMAX=seq(0,150,1); HILL=1; R0=0; AC50=seq(log2(0.00001), log2(100), 0.1)
conc = c(-10.99369, -8.671762, -6.349834, -5.188954, -4.027906, -2.866825, -1.705978, -0.5448964, 0.6159502, 1.77687, 2.937878, 4.098798, 5.259806, 6.259806)

Resp.sim=list(); NAMES=vector(); RMAX.val=vector(); AC50.val=vector(); x=1;
for(i in 1:length(AC50)){
    for(j in 1:length(RMAX)){
        Resp.sim[[x]] = R0 + (RMAX[j]-R0)/(1+2^((AC50[i]-conc) * HILL)) + rnorm(14, mean=0, sd=0);
        NAMES[x]=paste("AC50.", AC50[i], "_RMAX.", RMAX[j], "*", sep="")
        RMAX.val[x] = RMAX[j];
        AC50.val[x] = AC50[i];
        x=x+1;
    }
    if(i/1000 == as.integer(i/1000)){print(i)}
}

# put data into matrix
SimData.mat = matrix(NA, nrow=length(Resp.sim), ncol=14);
SimData.mat = data.frame(SimData.mat);
for(i in 1:length(Resp.sim)){
    SimData.mat[i,] = Resp.sim[[i]]; 
}

# calculate WES scores (use generic detection limit setting = 25%)
WES.fig2  = WES.FX(data=SimData.mat, RespCols=1:14, DetLimit=25)

# generate 3d plot using R/scatterplot3d (this package must be downloaded and installed)
# Ligges,U. and Maechler,M. (2003) Scatterplot3d  an R package for visualizing multivariate data. J. Stat. Softw., 8, 1-20.
library(scatterplot3d);

pos.tmp = which(SimData.mat[,1] == 0); #define values for RMAX = 0 (remove these values in the code below)
scatterplot3d(x=AC50.val[1:length(Resp.sim) %w/o% pos.tmp], y=RMAX.val[1:length(Resp.sim) %w/o% pos.tmp], z=WES.fig2[1:length(Resp.sim) %w/o% pos.tmp], pch=19, cex.symbols=0.6, xlab="AC50", ylab="|RMAX|", zlab="Weighted Entropy (WES)")





